
#include "os.h"

#include "top.h"
#include "colour.h"

#ifdef COLOUR

extern char *myname;
extern int colour_on;

static colour_entry_t *colour_array = NULL;
static int colour_array_size = 0;
static char **colour_names = NULL;

char *default_colours_env = "cpu=25,75#31:cpu=75,#31;1:size=50,100#35:size=100,#35;1:res=50,100#35:res=100,#35;1:1min=,100#32:1min=100,500#33:1min=500,#31;1:5min=,100#32:5min=100,500#33:5min=500,#31;1:15min=,100#32:15min=100,500#33:15min=500,#31;1:host=,#36";

/* These can be provided without the machine specific module being altered */
char *default_colour_names[] = 
  { "host", "1min", "5min", "15min", NULL };

/* colour_env_parse(env) - parse out colour settings from an environment
   variable.

   cpu=<min%>,<max%>#colour codes:
   size=<minK>,<maxK>#colour codes:
   res=<minK>,<maxK>#colour codes:
   time=<min_secs>,<max_secs>#colour codes

   Valid ANSI values are:

0	Reset all attributes
1	Bright
2	Dim
4	Underscore	
5	Blink
7	Reverse
8	Hidden

	Foreground Colours
30	Black
31	Red
32	Green
33	Yellow
34	Blue
35	Magenta
36	Cyan
37	White

	Background Colours
40	Black
41	Red
42	Green
43	Yellow
44	Blue
45	Magenta
46	Cyan
47	White

*/

#define COLOUR_REALLOC_INCREMENT 10

int colour_env_parse(env,c_names)
     char *env;
     char **c_names;
{
  char *entry,*min,*max,*col,*end;
  char entry_buffer[64];
  int len,type,array_index = 0;
  char **type_scan;
  int warnings = 0, no_warnings = 0;

  /* We preserve a pointer to the colour array, for get_colour(). If
   * none supplied, use the default, but dont flag up warnings.
   */  
  if (c_names) {
    colour_names = c_names;
  } else {
    colour_names = default_colour_names;
    no_warnings = 1;
  }

  if (!env || !*env) {
    env = default_colours_env;
    no_warnings = 1;
  }
	    
  end = env;
  do {
    entry = end;
    end = strchr(entry,':');
    len = sizeof(entry_buffer)>end-entry?end-entry:sizeof(entry_buffer);
    strncpy(entry_buffer, entry,len);
    entry_buffer[len] = '\000';
    if (end) end++;

    /* Parse entry_buffer */
    for (type=0,type_scan=colour_names ; *type_scan ; type_scan++,type++) {
      if (strncmp(entry_buffer,*type_scan,strlen(*type_scan)) == 0)
	break;
    }
    if (!*type_scan) {
      if (!no_warnings)
	fprintf(stderr, "Bad colour entry (unknown type): '%s'\n",
		entry_buffer);
      warnings++;
      continue;
    }
    if (!(min = strchr(entry_buffer,'='))) {
      if (!no_warnings)
	fprintf(stderr, "Bad colour entry: '%s'\n",entry_buffer);
      warnings++;
      continue;
    }
    min++;
    if (!(max = strchr(entry_buffer,','))) {
      if (!no_warnings)
	fprintf(stderr, "Bad colour entry: '%s'\n",entry_buffer);
      warnings++;
      continue;
    }
    max++;
    if (!(col = strchr(entry_buffer,'#'))) {
      if (!no_warnings)
	fprintf(stderr, "Bad colour entry: '%s'\n",entry_buffer);
      warnings++;
      continue;
    }
    col++;
    
    if (array_index >= (colour_array_size-1)) {
      colour_array_size += COLOUR_REALLOC_INCREMENT;
      colour_array = (colour_entry_t *)realloc(colour_array,
					       (colour_array_size*sizeof(colour_entry_t)));
      if (!colour_array) {
	fprintf(stderr, "%s: can't allocate colour table\n", myname);
	quit(1);
      }
    }
    colour_array[array_index].type = type;
    colour_array[array_index].min = atoi(min);
    colour_array[array_index].max = atoi(max);
    strncpy(colour_array[array_index].colour,col,
	    sizeof(colour_array[array_index].colour));
    array_index++;
  } while (end);
  if (colour_array)
    colour_array[array_index].type = -1;
  return (warnings>0 && !no_warnings?1:0);
}

char *stop_colour()
{
  if (!colour_on || !colour_array) return "";

  return "\033[0m";
}

/* get_colour - returns a start colour string for this type
   at this value.
*/
   
char *get_colour(type_str,value,buf,buf_size)
     char *type_str;
     int  value;
     char *buf;
     int  buf_size;
      
{
  int scan;
  int type;

  buf[0] = '\000';
  
  if (!colour_on || !colour_array ||
      (type = string_index(type_str,colour_names)) < 0)
    return buf;
  
  for (scan = 0 ; colour_array[scan].type != -1 ; scan++) {
    if (colour_array[scan].type == type &&
	(!colour_array[scan].min || colour_array[scan].min <= value) &&
	(!colour_array[scan].max || colour_array[scan].max >= value)) {
      if (strlen(colour_array[scan].colour) == 0)
	  return buf;
      snprintf(buf,buf_size,"\033[%sm",colour_array[scan].colour);
      return buf;
    }
  }
  return buf;
}
     
#endif
